<?php

//
// innovaphone fax call out PHP script
// copyright (c) innovaphone 2012 - 2015
//

$path=dirname(__FILE__);
require_once("$path/../include.php");
require_once("classes/translator.class.php");
require_once("$path/../app-classes/innovaphone.fax.config.class.php");
require_once("$path/../app-classes/innovaphone.fax.class.php");
require_once("$path/../app-classes/innovaphone.fax.shm.class.php");
require_once("$path/../app-classes/innovaphone.fax.database.class.php");

$version = file_get_contents("$path/../version");

declare(ticks = 1);
?>

innovaphone fax call out PHP script
copyright (c) innovaphone 2012 - 2014
<?php

print "Version " . $version . "\n\n";

$help = false;
foreach($argv as $arg) {
    if(in_array($arg, array('--help', '-help', '-h', '-?'))) {
        $help = true;
    }
}

if(!is_array($_GET) && ($argc < 3 || $help)) {
?>
    Usage:
  <?php echo $argv[0]; ?> <id> <jobGuid>

<id> is the configuration database id specific for this server instance.
<jobGuid> is the guid of the fax job which should be handled.

With the --help, -help, -h, or -? options, you can get this help.

<?php
exit(1);
} 

$configurationId        = null;
$jobGuid                = null;

if($argc >= 2) {
    $configurationId    = intval($argv[1]);
    $jobGuid            = $argv[2];
}
else if(is_array($_GET)) {
    $configurationId    = intval($_GET['id']);
    $jobGuid            = $_GET['jobGuid'];
}


setlocale(LC_CTYPE, "en_US.UTF-8");

function getLogDate() {
    $now = new DateTime();
    return date('Ymd-His') . "  -  ";
}

function sig_handler($nsig)
{
    print "\n";
    print getLogDate() . "terminated($nsig).\n";
    exit;
}

pcntl_signal(SIGTERM, "sig_handler", false);
pcntl_signal(SIGINT, "sig_handler", false);
pcntl_signal(SIGHUP, "sig_handler", false);


function terminateSession($pbx) {
    // terminate the session
    try {
        $p = $pbx->End($pbx->getSession());
    }
    catch(Exception $e) {
        return false;
    }
    return true;
}

function shutdown_function() {
    global $pbx;
    global $faxShm;
    global $jobGuid;
    if($faxShm && $jobGuid) {
        $faxShm->unlockJob($jobGuid);
    }
    if($pbx && $pbx->getKey()) {
        terminateSession($pbx);
    }
    print "\n";
}

register_shutdown_function('shutdown_function');

$pbx        = null;
$faxShm     = null;


print getLogDate() . "Reading configuration... ";
$cfg     = new FaxServer();
$cfg->read("", $configurationId);
if(!$cfg) {
    print "failed.\n";
    die("Could not read configuration.");
}
print "done.\n";


tl::change("en");
tl::register("faxserver", dirname(__FILE__) . "/../lang");
tl::change($cfg->lang);


print getLogDate() . "Attaching to shared memory... ";
$faxShm     = new InnovaphoneFaxShm($cfg->shmId);
if(!$faxShm->isAttached()) {
    print "failed.\n";
    die("Could not attach to the SHM.");
}
print "done.\n";


print getLogDate() . "Connecting to database... ";
$faxDatabase    = new InnovaphoneFaxDatabase();
if(!$faxDatabase->isConnected()) {
    print "failed.\n";
    die("Could not connect to the database.");
}
$faxDatabase->setServerId($cfg->id);
print "done.\n";


print getLogDate() . "Trying to lock job... ";
$ret = $faxShm->lockJob($jobGuid, null, null);
if(!$ret) {
    $faxShm = 0;
    print "failed.\n";
    die("Could not lock fax job.");
}
print "done.\n";


print getLogDate() . "Reading fax job database entry... ";
$faxJob     = $faxDatabase->getFaxJob($jobGuid);
if(!$faxJob) {
    print "failed.\n";
    die("Could not read database object.");   
}
print "done.\n";


if($faxJob && $faxJob->getDirection() != 'outgoing') {
    die("Fax job with wrong direction found.");
}

if($faxJob->getProgress() == 'queued' &&
    $faxJob->getResult() != 'success' &&
    $faxJob->getResult() != 'error')
{

    $complete   = false;

    $server     = $cfg->soapHost;
    $port       = $cfg->soapPort;
    $https      = $cfg->https;
    $httpu      = $cfg->soapUser;
    $httpp      = $cfg->soapPassword;
    $userName   = $cfg->soapFaxUser;
    $dynPbxId   = $cfg->soapDynPbx;
    $webdavUrl  = $cfg->webdavURL;
    
    print getLogDate() . "Connecting to PBX " . $server . "... ";
    $pbx = new InnovaphonePBX(
        $server,
        $port,
        $https,
        $httpu,
        $httpp,
        $userName,
        "innovaphone PHP fax call out [" . $version . "]",
        $cfg->trace ? array("trace" => 1) : null,
        dirname(__FILE__) . '/../wsdl/pbx10_00.wsdl',
        $dynPbxId);
    
    if($pbx->getKey() == 0) {
        print "failed.\n";
        die("\nFailed to login to PBX: " . $pbx->getFaultString() . "\n");
    }
    print "connected.\n\n";

    $pbx->showVersion();
    print "\n";

    try {
        $userHandle = $pbx->UserInitialize($pbx->getSession(), $userName, true, false, "");
        if(!$userHandle) {
            die("Can't subscribe to user {$userName}.\n");
        }
    }
    catch(Exception $e) {
        die("SOAP connection error.");
    }

    $fax = new InnovaphoneFax($pbx, $userName);
    $pbxInfo = &$fax;

    if(!$pbxInfo->initialize()) {
        continue;
    }

    $user = $pbxInfo->getUser($userName);
    if(!$user) {
        die("User is not in poll list.\n");
    }

    $pbxUser = $pbxInfo->findUserH323($faxJob->getUserH323());
    
    print getLogDate() . "Update job timestamp... ";
    $faxJob = $faxDatabase->updateFaxJob($faxJob);
    if(!$faxJob) {
        print "failed.\n";
        die("Could not update database object.");
    }
    print "done.\n";


    $webdavFileName                         = $faxJob->getFileName();
    $callerNumber                           = $pbxUser ? ($pbxUser->getSendNumber() != "" ? $pbxUser->getSendNumber() : $pbxUser->getNumber()) : $faxJob->getUserNumber();
    $destinationNumber                      = $faxJob->getRemoteNumber();
    $subscriberId                           = "";
    $restrictedNumber                       = strtoupper(substr($callerNumber, 0, 1)) == "R";
    

    if(!$restrictedNumber) {
        if($pbxUser && $pbxUser->getPseudoType() == "map") {
            $subscriberId                       = $cfg->faxSubscriberIdAlt;
            if($subscriberId != "") {
                $subscriberId .= strpos($subscriberId, ' ') !== false ? " " : "";
                $subscriberId .= $callerNumber;
            }
        }
        else {
            if(substr($callerNumber, 0, strlen($cfg->faxUserNumberPrefix)) == $cfg->faxUserNumberPrefix) {
                $callerNumber = substr($callerNumber, strlen($cfg->faxUserNumberPrefix) - strlen($callerNumber));
            }
            $subscriberId                       = $cfg->faxSubscriberId;
            if($subscriberId != "" && $cfg->faxAddUserNumber) {
                $subscriberId .= $callerNumber;
            }
            if($pbxUser != $user) {
                $callerNumber = ($user->getSendNumber() != "" ? $user->getSendNumber() : $user->getNumber()) . $callerNumber;
            }
        }
    }
    else {
        $callerNumber = "R";
    }    
    
    $headerData                             = array();
    $headerData['<subscriber-name>']        = $cfg->faxSubscriberName;
    $headerData['<user-name>']              = $faxJob->getUserName();
    $headerData['<user-h323>']              = $faxJob->getUserH323();
    $headerData['<user-number>']            = $faxJob->getUserNumber();     //the user phone number, not used
    $headerData['<remote-number>']          = $faxJob->getRemoteNumber();
    $headerData['<pages>']                  = $faxJob->getPages();
    $headerLineTextId                       = (strlen($cfg->faxSubscriberName) > 22) ? FaxServerStrings::msgHeaderLineLongCompanyName : FaxServerStrings::msgHeaderLine;
    $headerLine                             = $cfg->getString($headerLineTextId, $faxJob->getUserLanguage());
    $headerLine                             = strtr($headerLine, $headerData);
    
    
    print getLogDate() . "Calling to '" . $destinationNumber . "' with '" . $callerNumber . "'... \n";
    $callHandle = $fax->makeCall($userHandle, $callerNumber, $destinationNumber);

    $callState          = 0;
    $call               = 0;
    $page               = 0;
    $connectedTime      = null;
    $responseTimedOut   = false;
    $responseTimeout    = new DateTime();
    $responseTimeout->modify("+ " . $cfg->faxResponseTimeout . " seconds");

    if($callHandle) {
        do {
            $pbxInfo->purge();
            
            try {
                $p = $pbx->Poll($pbx->getSession());
            }
            catch(Exception $e) {
                try {
                    $p = $pbx->Echo($pbx->getSession(), $pbx->getKey());
                }
                catch(Exception $e) {
                    $pbx = 0;
                    die("Connection error: " . $e->faultstring . "\n");
                }
                if($p) {
                    continue;
                }
                else {
                    $pbx = 0;
                    print getLogDate() . "Reconnecting and trying again...\n\n";
                    break;
                }
            }

            $pbxInfo->update($p);
            $call = $fax->getCall($callHandle);
            
            if($call) {
                $update = $fax->updateOutgoingCall($call->getCallHandle(), $webdavFileName, $subscriberId, $headerLine, $cfg->trace, $cfg->modemSpeed);
                $faxShm->updateJob($jobGuid, $call->getConfId(), null);
                $faxShm->updateCall($call);
                if($update || $page != $call->getPage()) {
                    print getLogDate() . $call->getFaxState();
                    if($call->getPage()) {
                        print ", page " . $call->getPage();
                    }
                    $page = $call->getPage();
                    if($call->getErrorCause()) {
                        print ", " . $call->getErrorCause();
                    }
                    if($call->getErrorDescription()) {
                        print ", " . $call->getErrorDescription();
                    }
                    print "...\n";
                    if($call->getFaxState() == 'connected') {
                        $responseTimeout = null;
                    }
                }
                if(!$connectedTime && ($call->getState() & 0xF) == 5) {
                    $connectedTime = new DateTime();
                    $faxJob->setCallTimestamp($connectedTime);
                    $timestampJob = $faxDatabase->updateFaxJob($faxJob);
                    if(!$timestampJob) {
                        print getLogDate() . "Could not set call timestamp.\n";
                    }
                }
            }
            if($responseTimeout) {
                $now = new DateTime();
                if($responseTimeout <= $now) {
                    $responseTimedOut = true;
                    $responseTimeout = 0;
                    print getLogDate() . "Response timeout.\n";
                    if($call) {
                        $pbx->UserClear($call->getCallHandle(), 0x10/*Q931_CAUSE_NormalCallClearing*/, array());
                    }
                }
            }
            
        } while((!$call || $call->isActive()));
        
        print getLogDate() . "Call disconnected.\n";
        $faxShm->updateJob($jobGuid, null, null);
    }
    else {
        print "Could not create PBX user call.\n";
    }

    if($pbx) {
        terminateSession($pbx);
        $pbx = 0;
    }

    $faxJob->setErrorCause(null);
    $faxJob->setErrorDescription(null);
    
    if($call && !$responseTimedOut) {
        if($call->getRemoteSubscriberId()) {
            $faxJob->setRemoteSubscriberId($call->getRemoteSubscriberId());
        }

        switch($call->getFaxState()) {
            case 'aborted':
            case 'error':
                $faxJob->setResult($call->getFaxState());
                if($call->getErrorCause())          $faxJob->setErrorCause($call->getErrorCause());
                if($call->getErrorDescription())    $faxJob->setErrorDescription($call->getErrorDescription());
                break;    

            case 'completed':
                $faxJob->setResult('success');
                $faxJob->setPages($call->getPages());
                if($connectedTime) {
                    $now = new DateTime();
                    $duration = $now->diff($connectedTime);
                    $faxJob->setDuration($duration);
                    $faxJob->setCallTimestamp($connectedTime);
                }
                break;    
            
            default:
                $faxJob->setResult('error');
                $faxJob->setErrorCause("script error");
                $faxJob->setErrorDescription("wrong fax call state");
                break;
        }
    }
    else if($call) {
        $faxJob->setResult('aborted');
        if($connectedTime) {
        $faxJob->setErrorCause("no fax device");
        }
        else {
            $faxJob->setErrorCause("response timeout");
        }
    }
    else if(!$callHandle) {
        $faxJob->setResult('aborted');
        $faxJob->setErrorCause("call aborted");
        $faxJob->setErrorDescription("not created in PBX");
    }
    else if($responseTimedOut) {
        $faxJob->setResult('aborted');
        $faxJob->setErrorCause("call aborted");
        $faxJob->setErrorDescription("not found in PBX");
    }
    else {
        $faxJob->setResult('aborted');
        $faxJob->setErrorCause("call aborted");
        $faxJob->setErrorDescription("unexpected call end");
    }

    if(!$callHandle || ($call &&
       ($call->getCauseCode() == 0x22/*Q931_CAUSE_NoCircuit_ChannelAvailable*/ ||
       $call->getCauseCode() == 0x26/*Q931_CAUSE_NetworkOutOfOrder*/ ||
       $call->getCauseCode() == 0x29/*Q931_CAUSE_TemporaryFailure*/ ||
       $call->getCauseCode() == 0x2A/*Q931_CAUSE_SwitchingEquipmentCongestion*/ ||
       $call->getCauseCode() == 0x2C/*Q931_CAUSE_RequestedCircuit_ChannelNotAvailable*/ ||
       $call->getCauseCode() == 0x2F/*Q931_CAUSE_ResourcesUnvailable_Unspecified*/ ||
       $call->getCauseCode() == 0x31/*Q931_CAUSE_QualityOfServiceUnavailable*/))) {
        $now             = new DateTime();
        $faxJob->setCalls($faxJob->getCalls() != "" ? $faxJob->getCalls() : 0);
        if(!$faxJob->getCallTimestamp()) {
            $faxJob->setCallTimestamp($now);
        }
        if($faxJob->getCallTimestamp() > date_modify(new DateTime(), "- " . $cfg->faxRetryTimeout . " minutes")) {
            $faxJob->setDelayed(date_modify(new DateTime(), "+ 1 minute"));
        }
        else {
            $faxJob->setResult('error');
            $faxJob->setErrorDescription("retry timeout");
            $faxJob->setDelayed(null);
        }
    }
    else {
        $faxJob->setCalls($faxJob->getCalls() + 1);

        if($faxJob->getResult() == 'aborted') {
            if($faxJob->getCalls() < $cfg->faxMaxCallTries) {
                $now             = new DateTime();
                $now->modify("+ " . $cfg->faxResendDelay . " minutes");
                $faxJob->setDelayed($now);
            }
            else {
                $faxJob->setResult('error');
                $faxJob->setDelayed(null);
            }
        }
        else {
            $faxJob->setDelayed(null);
        }
    }

    print getLogDate() . "Updating database... ";
    $faxJob = $faxDatabase->updateFaxJob($faxJob);
    if(!$faxJob) {
        print "failed.\n";
        die("Can't update database object.");
    }
    print "done.\n";
    
}

$faxShm->unlockJob($jobGuid);

print getLogDate() . "Finished.\n";
?>
